#
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#

package SnapMirrorSnap;

use NaServer;

use strict;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);

@EXPORT = qw();

$VERSION = '1.00';

require 5.000;
use ag_i18n_inc;

my $vcs_home = $VCSHOME;

%default = {
	resname       => '',
	targetresname => '',
	filername     => '',
	sotype        => '',
	volname       => '',
};

my $lockfile_prefix =
  "$vcs_home" . "/bin/NetAppSnapMirrorSnap/.VCS_SnapMirrorSnap_";
my $lockfile_suffix = "_Online_Lockfile.DO-NOT-REMOVE";

sub new {
	my $proto = shift;
	my (
		$resname, $targetresname, $filername, $filerpword,
		$mountSO, $useSO,         $requireSO, $snapshotName
	  )
	  = @_;
	my ( $group, $basedir );
	my $class = ref($proto) || $proto;

	VCSAG_LOGDBG_MSG( 1,
		"new(): resname $resname,targetresname: $targetresname,filername: $filername, filerpword:$filerpword, m:$mountSO, u:$useSO, r:$requireSO "
	);
	my $self = {%default};
	$self->{'resname'}       = $resname;
	$self->{'targetresname'} = $targetresname;
	$self->{'filername'}     = $filername;
	$self->{'snapname'}      = $snapshotName;

	if ($mountSO) {
		if ($requireSO) {
			$self->{'sotype'} = "gold-only";
		} else {
			$self->{'sotype'} = 'gold';
		}
	} elsif ($useSO) {
		if ($requireSO) {
			$self->{'sotype'} = "silver-only";
		} else {
			$self->{'sotype'} = 'silver';
		}
	} else {
		$self->{'sotype'} = "bronze";
	}

	$self->{'volname'} = `$HARES -value $targetresname VolumeName`;

	if ( $? != 0 ) {
		VCSAG_LOG_MSG( "W",
			"Did not get VolumeName attribute from resource $targetresname !",
			3001, $targetresname );
		return undef;
	}
	chomp $self->{'volname'};

	# Open  a persistent connection to the filer.

	my $f_handle = NaServer->new( $filername, 1, 3 );

	# Strip leading and trailing spaces from encrypted password
	$filerpword =~ s/^\s+//;
	$filerpword =~ s/\s+$//;

	# If a password exists use it else use hosts authentication
	if ($filerpword) {
		my $dec_pword = `$HADECRYPT $filerpword`;
		if ( $? != 0 ) {
			VCSAG_LOG_MSG( "W", "Encountered errors while decrypting password!",
				3002 );
			$dec_pword = "";
		}

		$f_handle->set_style('LOGIN_PASSWORD');
		$f_handle->set_admin_user( "root", "$dec_pword" );

	} else {
		$f_handle->set_style('HOSTSEQUIV');
	}

	my $output = $f_handle->invoke("system-get-version");
	if ( $output->results_status() eq "failed" ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"ONTAPI 'system-get-version' failed on filer $filername. Error $output->results_errno: $reason",
			3003, $filername, $output->results_errno, $reason );
		return undef;
	}

	$self->{'handle'} = $f_handle;

	bless( $self, $class );
	return $self;
}

sub DESTROY { }

#
# Check whether a service is licensed on the filer
# Demo licenses are not considered
#
sub CheckLicense {
	my ( $self, $service ) = @_;
	my ( $output, $str, $lic, $islic, $retval );
	my $filerhandle = $self->{'handle'};

	$output = $filerhandle->invoke("license-list-info");
	my $license_list = $output->child_get("licenses");
	my @licenses     = $license_list->children_get();

	foreach $lic (@licenses) {
		$str = $lic->child_get_string("service");
		next if ( $service ne $str );
		$islic = $lic->child_get_string("is-licensed");
		last;
	}

	$retval = ( $islic =~ /false/i ) ? 0 : 1;
	return $retval;
}

sub create_lockfile {
	my ($self) = @_;
	my $LockFile = $lockfile_prefix . $self->{'resname'} . $lockfile_suffix;
	open( lfh, ">$LockFile" )
	  || VCSAG_LOG_MSG( "E", "Could not create lock $LockFile : $!",
		3004, $LockFile, $! );
	close(lfh);
}

sub remove_lockfile {
	my ($self) = @_;
	my $LockFile = $lockfile_prefix . $self->{'resname'} . $lockfile_suffix;
	unlink("$LockFile");
}

sub lockfile_exists {
	my ($self) = @_;
	my $LockFile = $lockfile_prefix . $self->{'resname'} . $lockfile_suffix;
	if ( -f "$LockFile" ) {
		return 1;
	} else {
		return 0;
	}
}

sub pitc {
	my ($self) = @_;
	my $ret = 0;
	if ( $self->{'sotype'} =~ /gold/ ) {
		$ret = $self->pitc_gold();
	} elsif ( $self->{'sotype'} =~ /silver/ ) {
		$ret = $self->pitc_silver();
	} else {
		my $split = 0;
		$ret = $self->pitc_bronze($split);
	}
	return $ret;
}

# In the 'gold' scenario, if the SnapshotName attribute is defined then we use 
# the snaphsot to create the clone.
# Else we quiesce and break and then create the clone.

sub pitc_gold {
	my ($self)  = @_;
	my $ret     = 0;
	my $snapret = 0;

	my $vol = $self->{'volname'};

	if ( $self->{'snapname'} eq "" ) {

		# vol clone needs a writable parent vol. So quiesce and break first
		$ret = $self->split_replication();
		if ($ret) {
			VCSAG_LOG_MSG( "E",
				"Could not split the replication in pitc_gold for ${vol}",
				3028, ${vol} );
			return 1;
		}

	}

	$snapret = $self->take_snapshot();

	if ($snapret) {
		VCSAG_LOG_MSG( "E", "Cannot take required clone of volume: ${vol} ",
			3006, ${vol} );
		if ( $self->{'sotype'} eq "gold-only" ) {
			return 1;
		} else {
			$ret = $self->pitc_bronze();
			return $ret;
		}
	} else {
		my (@fields) = split( '\s+', `$HASYS -nodeid` );
		system("$HARES -modify $self->{'resname'} Responsibility $fields[5]");
	}
	$ret = $self->mount_snapshot("gold");

	return $ret;
}

sub pitc_silver {
	my ($self)  = @_;
	my $ret     = 0;
	my $snapret = 0;

	my $vol = $self->{'volname'};

	# vol clone needs a writable parent vol. So quiesce and break first
	$ret = $self->split_replication();
	if ($ret) {
		VCSAG_LOG_MSG( "E",
			"Could not split the replication in pitc_silver for ${vol}",
			3008, ${vol} );
		return 1;
	}

	$snapret = $self->take_snapshot();

	if ($snapret) {
		VCSAG_LOG_MSG( "E", "Cannot take required clone of volume: ${vol}",
			3006, ${vol} );
		if ( $self->{'sotype'} eq "silver-only" ) {
			return 1;
		} else {
			$ret = $self->pitc_bronze();
			return $ret;
		}
	}

	$ret = $self->mount_snapshot("silver");

	return $ret;

}

# Point in time copy -- Bronze. Just split the replication and add an nfs entry
# into the exports file

sub pitc_bronze {
	my ($self)  = @_;
	my $ret     = 0;
	my $snapret = 0;

	my $vol = $self->{'volname'};

#
# vol clone needs a writable parent vol. So quiesce and break first
# If this is a fallback from gold or silver, then the replication was already 
# split. There is a special case if this was "gold" with a snapshot name 
# specified. In that case the replication has not been split and we need to 
# handle it here
#
	if (
		( $self->{'sotype'} eq "bronze" )
		|| (
			(
				   ( $self->{'sotype'} eq "gold" )
				|| ( $self->{'sotype'} eq "gold-only" )
			)
			&& ( $self->{'snapname'} ne "" )
		)
	  )
	{
		$ret = $self->split_replication();
		if ($ret) {
			VCSAG_LOG_MSG( "E",
				"Could not split the replication in pitc_bronze for ${vol}",
				3027, ${vol} );
			return 1;
		}

	}
	$ret = $self->mount_snapshot("bronze");

	return $ret;
}

# Quiesce and break the replication
sub split_replication {
	my ($self) = @_;
	my $ret = 0;

	#Quiesce
	my $ret = $self->SnapMirrorQuiesce( $self->{'volname'} );
	if ($ret) {
		VCSAG_LOG_MSG( "E",
			"Failed to quiesce destination location $self->{'volname'}  on filer $self->{'filername'}",
			3009, $self->{'volname'}, $self->{'filername'} );
		return 1;
	}

	#Break the replication
	my $ret = $self->SnapMirrorBreak( $self->{'volname'} );
	if ($ret) {
		VCSAG_LOG_MSG( "E",
			"Failed to break destination location $self->{'volname'}  on filer $self->{'filername'}",
			3010, $self->{'volname'}, $self->{'filername'} );
		return 1;
	}
	return $ret;
}

# Use this to add an entry to the /etc/exports file on the filer
# We add an entry of the form
# "/vol/<volname>_fd  -actual=/vol/<real vol>,sec=sys,rw=<nodenames>,root=<nodenames>"
# for each node in the cluster
sub mount_snapshot {
	my ( $self, $case ) = @_;
	my $ret            = 0;
	my $vol            = $self->{'volname'};
	my $nfs_entry_name = "/vol/${vol}_fd";
	my @nodelist       = ();
	my $f_handle       = $self->{'handle'};
	my @syslist;
	my $actual_entry_name = "";

	if ( ( $case eq "silver" ) || ( $case eq "bronze" ) ) {
		$actual_entry_name = "/vol/${vol}";
	} elsif ( $case eq "gold" ) {
		$actual_entry_name = "/vol/cl_${vol}";
	}

	my $tmpvar   = `$HASYS -nodeid `;
	my $nodename = ( split / +/, $tmpvar )[5];

	push( @nodelist, $nodename );

	$self->del_nfs_path($nfs_entry_name);

	$ret =
	  $self->add_nfs_path( $nfs_entry_name, $actual_entry_name, @nodelist );

	return $ret;
}

sub del_nfs_path {
	my $self           = shift;
	my $nfs_entry_name = shift;

	my $ret      = 0;
	my $f_handle = $self->{'handle'};
	my $output;

	my $input  = new NaElement("nfs-exportfs-delete-rules");
	my $p_info = new NaElement("pathname-info");

	my $pathnames = new NaElement("pathnames");
	$pathnames->child_add($p_info);

	$input->child_add_string( "persistent", "true" );
	$input->child_add($pathnames);
	$p_info->child_add_string( "name", "$nfs_entry_name" );

	$output = $f_handle->invoke_elem($input);
	if ( $output->results_status() eq "failed" ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "I",
			"Pathname \'$nfs_entry_name\' not found in nfs exports:  $reason",
			3029, $nfs_entry_name, $reason );
		$ret = 1;
	}

	return $ret;
}

sub add_nfs_path {
	my $self              = shift;
	my $nfs_entry_name    = shift;
	my $actual_entry_name = shift;
	my (@nodes)           = @_;
	my $ret               = 0;
	my $f_handle          = $self->{'handle'};
	my $output;

	my $input = new NaElement("nfs-exportfs-append-rules");
	my $rules = new NaElement("exports-rule-info");
	my $rule  = new NaElement("rules");

	$rule->child_add($rules);

	$input->child_add_string( "persistent", "true" );
	$input->child_add($rule);

	$rules->child_add_string( "pathname",        "$nfs_entry_name" );
	$rules->child_add_string( "actual-pathname", "$actual_entry_name" );

	my $rw = new NaElement("read-write");

	my $ehi;

	foreach my $host (@nodes) {
		$ehi = new NaElement("exports-hostname-info");
		$ehi->child_add_string( "name", $host );
		$rw->child_add($ehi);
	}

	$rules->child_add($rw);

	my $rt = new NaElement("root");

	foreach my $host (@nodes) {
		$ehi = new NaElement("exports-hostname-info");
		$ehi->child_add_string( "name", $host );
		$rt->child_add($ehi);
	}

	$rules->child_add($rt);

	$output = $f_handle->invoke_elem($input);
	if ( $output->results_status() eq "failed" ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E", "Cannot add rule to exports:  $reason",
			3026, $reason );
		$ret = 1;
	}

	return $ret;
}

# Take a vol clone of the volume
sub take_snapshot {
	my ($self) = @_;
	my $ret = 0;
	my $output;

	my $f_handle = $self->{'handle'};
	my $vol      = $self->{'volname'};

# Handle the special 'gold' case where user wants the clone from an existing 
# snapshot.

	if (
		(
			   ( $self->{'sotype'} eq "gold" )
			|| ( $self->{'sotype'} eq "gold-only" )
		)
		&& ( $self->{'snapname'} ne "" )
	  )
	{

		$output =
		  $f_handle->invoke( "volume-clone-create", "parent-volume", $vol,
			"volume", "cl_${vol}", "parent-snapshot", $self->{'snapname'} );
	} else {
		$output =
		  $f_handle->invoke( "volume-clone-create", "parent-volume", $vol,
			"volume", "cl_${vol}" );
	}

	if ( $output->results_status() eq "failed" ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"Cannot take 'vol clone' of ${vol} to cl_${vol}: $reason",
			3011, ${vol}, ${vol}, $reason );
		$ret = 1;
	}

	return $ret;
}

sub SnapMirrorQuiesce {
	my ( $self, $dstloc ) = @_;

	my $filername = $self->{'filername'};
	my $handle    = $self->{'handle'};

	#
	# wait for status to become idle or (TBD: timeout)
	#
	my $ret = $self->WaitForIdle($dstloc);
	if ($ret) {
		VCSAG_LOG_MSG( "E",
			"Error while waiting for snapmirror status of volume $dstloc to become 'idle' on filer $filername",
			3012, "$dstloc", "$filername" );
		return 1;
	}

	VCSAG_LOG_MSG( "N", "Attempting to quiesce snapmirror location $dstloc",
		3013, "$dstloc" );

	my $output =
	  $handle->invoke( "snapmirror-quiesce", "destination-location", $dstloc );

	if ( $output->results_errno != 0 ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"ONTAPI 'snapmirror-quiesce' failed for volume $dstloc on filer $filername. Error $output->results_errno: $reason",
			3014, $dstloc, $filername, $output->results_errno, $reason );
		return 1;
	}

	#
	# check if volume was successfully quiesced
	#
	my @sminfolist = $self->SnapMirrorGetInfo($dstloc);
	return 1 if ( !@sminfolist );

	my $sm_info = $sminfolist[0];
	if ( $sm_info->{STATE} ne "quiesced" ) {
		$ENV{"LANG"} = "$self->{LANG}";
		VCSAG_LOG_MSG( "E",
			"snapmirror-quiesce failed to quiesce the destination location $dstloc at filer $filername",
			3015, "$dstloc", "$filername" );
		$ENV{"LANG"} = "C";
		return 1;
	}

	VCSAG_LOG_MSG( "I",
		"snapmirrored location $dstloc has been successfully quiesced",
		3017, "$dstloc" );
	$ENV{"LANG"} = "C";

	return 0;
}

#
# Breaks a snapmirrored relationship on this filer for specified volume.
# Also queries status afterward to see if the volume was successfully
# broken-off
#
sub SnapMirrorBreak {
	my ( $self, $dstloc ) = @_;

	my $filername = $self->{'filername'};
	my $handle    = $self->{'handle'};

	VCSAG_LOG_MSG( "N", "Attempting to break snapmirror location $dstloc",
		3018, "$dstloc" );

	my $output =
	  $handle->invoke( "snapmirror-break", "destination-location", $dstloc );

	if ( $output->results_errno != 0 ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"ONTAPI 'snapmirror-break' failed for volume $dstloc on filer $filername. Error $output->results_errno: $reason",
			3019, $dstloc, $filername, $output->results_errno, $reason );
		return 1;
	}

	#
	# check if volume was successfully broken-off
	#
	my @sminfolist = $self->SnapMirrorGetInfo($dstloc);
	return 1 if ( !@sminfolist );

	my $sm_info = $sminfolist[0];
	if ( $sm_info->{STATE} ne "broken-off" ) {
		VCSAG_LOG_MSG( "E",
			"snapmirror-break failed to break snapmirror relationship for destination location $dstloc at filer $filername",
			3020, "$dstloc", "$filername" );
		return 1;
	}
	VCSAG_LOG_MSG( "I",
		"snapmirrored location $dstloc has been successfully broken-off",
		3021, "$dstloc" );

	return 0;
}

sub SnapMirrorGetInfo {
	my ( $self, $location ) = @_;

	my $filername = $self->{'filername'};
	my $handle    = $self->{'handle'};

	my $output =
	  $handle->invoke( "snapmirror-get-status", "location", $location );

	if ( $output->results_errno != 0 ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"ONTAPI 'snapmirror-get-status' failed for volume $location on filer $filername. Error $output->results_errno: $reason",
			3022, $location, $filername, $output->results_errno, $reason );
		return undef;
	}

	$self->{SNAPMIRROR_ON} = $output->child_get_string("is-available");

	# get snapmirror status
	my $statuslist = $output->child_get("snapmirror-status");
	if ( !defined($statuslist) || ( $statuslist eq "" ) ) {

		# no snapmirror status to report
		$ENV{"LANG"} = "$self->{LANG}";
		VCSAG_LOG_MSG( "E",
			"Volume $location on filer $filername is not a snapmirror replicated volume",
			3023, "$location", "$filername" );
		$ENV{"LANG"} = "C";
		return undef;
	}

	# iterate through snapmirror status info list.
	# NOTE: THIS AGENT CURRENTLY SUPPORTS ONLY ONE DESTINATION PER
	# SOURCE VOLUME
	my @infolist   = $statuslist->children_get();
	my @sminfolist = ();
	foreach my $sm_info (@infolist) {
		my $info = {};
		$info->{SRC_LOC} = $sm_info->child_get_string("source-location");
		( $info->{SRC_FLR}, $info->{SRC_VOL} ) = split( /:/, $info->{SRC_LOC} );
		$info->{DST_LOC} = $sm_info->child_get_string("destination-location");
		( $info->{DST_FLR}, $info->{DST_VOL} ) = split( /:/, $info->{DST_LOC} );
		$info->{STATE}     = $sm_info->child_get_string("state");
		$info->{STATUS}    = $sm_info->child_get_string("status");
		$info->{BASE_SNAP} = $sm_info->child_get_string("base-snapshot");
		$info->{PROGRESS}  = $sm_info->child_get_int( "transfer-progress", 0 );
		$info->{LAG}       = $sm_info->child_get_int( "lag-time", 0 );
		push( @sminfolist, $info );

		$ENV{"LANG"} = "$self->{LANG}";
		VCSAG_LOGDBG_MSG(
			4,
"Snapmirror info for volume $location on filer $filername: Source Location = $info->{SRC_LOC}, Destination Location = $info->{DST_LOC}, State = $info->{STATE}, Status = $info->{STATUS}, Base Snapshot = $info->{BASE_SNAP}, Progress = $info->{PROGRESS}, Lag = $info->{LAG}",
			"$location",
			"$filername",
			"$info->{SRC_LOC}",
			"$info->{DST_LOC}",
			"$info->{STATE}",
			"$info->{STATUS}",
			"$info->{BASE_SNAP}",
			"$info->{PROGRESS}",
			"$info->{LAG}"
		);
		$ENV{"LANG"} = "C";
	}
	return (@sminfolist);
}

#
# This routine will wait in a tight loop for the snapmirror status
# of the specified volume to become 'idle'. This is required because
# certain snapmirror commands can only be issued when the status is
# idle or else the command will fail.
#
sub WaitForIdle {
	my ( $self, $location ) = @_;

	my @sminfolist;

	my $filername = $self->{'filername'};
	my $handle    = $self->{'handle'};

	# wait for snapmirror status to be "idle"
	while (1) {
		@sminfolist = $self->SnapMirrorGetInfo($location);
		return 1 if ( !@sminfolist );

		my $sm_info = $sminfolist[0];
		last
		  if ( $sm_info->{STATUS} eq "idle"
			|| $sm_info->{STATUS} eq "in-sync" );
		undef(@sminfolist);

		# sleep 5 seconds before checking again
		sleep 5;

	}

	return 0;
}

sub resynch {
	my ($self) = @_;
	my $ret = 0;
	if ( $self->{'sotype'} =~ /gold/ ) {
		$ret = $self->resynch_gold();
	} elsif ( $self->{'sotype'} =~ /silver/ ) {
		$ret = $self->resynch_silver();
	} else {
		$ret = $self->resynch_bronze();
	}
	return $ret;
}

sub resynch_gold {
	my ($self) = @_;
	my $ret = 0;
	my $output;
	my $reason;
	my $f_handle = $self->{'handle'};
	my $vol      = $self->{'volname'};

	my $nfs_entry_name = "/vol/${vol}_fd";

   # This might be getting called from a clean after an unsuccessful 'vol clone'
	$output = $f_handle->invoke( "volume-list-info", "volume", "cl_${vol}" );
	if ( $output->results_status() eq "failed" ) {
		$reason = $output->results_reason();
		VCSAG_LOG_MSG( "W",
			"Could not get status of volume cl_${vol}: $reason ",
			3032, ${vol}, $reason );
	} else {

		$output = $f_handle->invoke( "volume-offline", "name", "cl_${vol}" );
		if ( $output->results_status() eq "failed" ) {
			$reason = $output->results_reason();
			VCSAG_LOG_MSG( "E", "Could not offline volume cl_${vol}: $reason ",
				3030, ${vol}, $reason );
			return 1;
		}
		$output = $f_handle->invoke( "volume-destroy", "name", "cl_${vol}" );
		if ( $output->results_status() eq "failed" ) {
			$reason = $output->results_reason();
			VCSAG_LOG_MSG( "E", "Could not destroy volume cl_${vol}: $reason",
				3031, ${vol}, $reason );
			return 1;
		}
	}
	if ( $self->{'snapname'} eq "" ) {

		$output =
		  $f_handle->invoke( "snapmirror-resync", "destination-location",
			$vol );
		if ( $output->results_status() eq "failed" ) {
			$reason = $output->results_reason();
			VCSAG_LOG_MSG( "E",
				"Cannot resync snapmirror for volume $vol :$reason ",
				3024, ${vol}, $reason );
			$ret = 1;
		}
	}

	$self->del_nfs_path($nfs_entry_name);

	return $ret;
}

# resync and resume replication
# Destroy the clone that we created before resync'ing
sub resynch_silver {
	my ($self) = @_;
	my $ret = 0;
	my $output;
	my $reason;

	my $f_handle = $self->{'handle'};
	my $vol      = $self->{'volname'};

	my $nfs_entry_name = "/vol/${vol}_fd";

   # This might be getting called from a clean after an unsuccessful 'vol clone'
	$output = $f_handle->invoke( "volume-list-info", "volume", "cl_${vol}" );
	if ( $output->results_status() eq "failed" ) {
		$reason = $output->results_reason();
		VCSAG_LOG_MSG( "W",
			"Could not get status of volume cl_${vol}: $reason ",
			3032, ${vol}, $reason );
	} else {

		$output = $f_handle->invoke( "volume-offline", "name", "cl_${vol}" );
		if ( $output->results_status() eq "failed" ) {
			$reason = $output->results_reason();
			VCSAG_LOG_MSG( "E", "Could not offline volume cl_${vol}: $reason ",
				3030, ${vol}, $reason );
			return 1;
		}
		$output = $f_handle->invoke( "volume-destroy", "name", "cl_${vol}" );
		if ( $output->results_status() eq "failed" ) {
			$reason = $output->results_reason();
			VCSAG_LOG_MSG( "E", "Could not destroy volume cl_${vol}: $reason",
				3031, ${vol}, $reason );
			return 1;
		}
	}
	$output =
	  $f_handle->invoke( "snapmirror-resync", "destination-location", $vol );
	if ( $output->results_status() eq "failed" ) {
		$reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"Cannot resync snapmirror for volume $vol :$reason ",
			3024, ${vol}, $reason );
		$ret = 1;
	}

	$self->del_nfs_path($nfs_entry_name);

	return $ret;
}

# resync and resume replication
sub resynch_bronze {
	my ($self) = @_;
	my $ret = 0;

	my $f_handle = $self->{'handle'};
	my $vol      = $self->{'volname'};
	my $output   =
	  $f_handle->invoke( "snapmirror-resync", "destination-location", $vol );

	if ( $output->results_status() eq "failed" ) {
		my $reason = $output->results_reason();
		VCSAG_LOG_MSG( "E",
			"Cannot resync snapmirror for volume $vol : $reason ",
			3025, ${vol}, $reason );
		$ret = 1;
	}

	return $ret;
}

1;
